// naumann@stce.rwth-aachen.de

#include<iostream>
#include<fstream>
#include<cassert>
#include<cmath>

/// static vector type and arithmetic
template<int N, typename T>
class vector {
  T vals[N];
public:
  /// constructor initializes to zero
  vector() {
    for (size_t i=0;i<N;i++) vals[i]=T(0);
  }
  /// constructor initializes to val
  vector(const T& val) {
    for (size_t i=0;i<N;i++) vals[i]=T(val);
  }

  /// copy constructor
  vector(const vector &v) { 
    for (size_t i=0;i<N;i++) vals[i]=v.vals[i]; 
  }

  /// assignment
  vector& operator=(const vector &v) { 
    if (&v!=this) 
      for (size_t i=0;i<N;i++) vals[i]=v.vals[i]; 
    return *this;
  }

  /// read/write element access
  T& operator[](const size_t i) { 
    std::cout << "non-const []" << std::endl;
    assert(i>=0&&i<N); 
    return vals[i]; 
  }

  /// read-only element access
  const T& operator[](const size_t i) const { 
    std::cout << "const []" << std::endl;
    assert(i>=0&&i<N); 
    return vals[i]; 
  }

  /// l1_norm as member function
  T l1_norm() const {
    T n=0;
    for (int i=0;i<N;i++) n+=fabs(vals[i]);
    return n;
  }

  /// binary addition as member function
  vector operator+(const vector& v) const {
    vector<N,T> w;
    for (size_t i=0;i<N;i++) w.vals[i]=vals[i]+v.vals[i];
    return w;
  }

  /// binary multiplication as member function
  vector operator*(const vector& v) const {
    vector<N,T> w;
    for (size_t i=0;i<N;i++) w.vals[i]=vals[i]*v.vals[i];
    return w;
  }

  /// unary minus as member function
  vector operator-() const { 
    vector<N,T> v;
    for (size_t i=0;i<N;i++) v.vals[i]=-vals[i];
    return v;
  }
  /// incremental assignment as member function
  vector& operator+=(const vector& v) {
    for (size_t i=0;i<N;i++) vals[i]+=v.vals[i];
    return *this;
  }

  /// prefix increment as member function
  vector& operator++() {
    for (size_t i=0;i<N;i++) vals[i]++;
    return *this;
  }

  /// postfix decrement as member function
  vector operator--(int) {
    vector w(*this);
    for (size_t i=0;i<N;i++) vals[i]--;
    return w;
  }
  /// binary relational operator as member function
  bool operator<(const vector& v) const {
    if (l1_norm()<v.l1_norm()) return true;
    return false;
  }

  /// print to screen
  void print() const {
    std::cout << "[ ";
    for (size_t i=0;i<N;i++) std::cout << vals[i] << " ";
    std::cout << "]" << std::endl;
  }
  // non-member friends
  template<int M, typename S>
  friend S l2_norm(const vector<M,S>&);
  template<int M, typename S>
  friend vector<M,S> sin(const vector<M,S>&);
  template<int M, typename S>
  friend const vector<M,S>& max(const vector<M,S>&, const vector<M,S>&);
  template<int M, typename S>
  friend vector<M,S> operator-(const vector<M,S>&, const vector<M,S>&);
  template<int M, typename S>
  friend vector<M,S> operator/(const vector<M,S>&, const vector<M,S>&);
  template<int M, typename S>
  friend vector<M,S>& operator-=(vector<M,S>&, const vector<M,S>&);
  template<int M, typename S>
  friend vector<M,S>& operator--(vector<M,S>&);
  template<int M, typename S>
  friend vector<M,S> operator++(vector<M,S>&,int);
  template<int M, typename S>
  friend std::ostream& operator<<(std::ostream&, const vector<M,S>&);
  template<int M, typename S>
  friend std::istream& operator>>(std::istream&, vector<M,S>&);
};

/// l2-norm of vector as non-member function
template<int N, typename T>
T l2_norm(const vector<N,T> &v) {
  T n=0;
  for (int i=0;i<N;i++) n+=pow(v.vals[i],2);
  return sqrt(n);
}

/// sine as non-member function
template<int N, typename T>
vector<N,T> sin(const vector<N,T>& v) {
  vector<N,T> w;
  for (size_t i=0;i<N;i++) w.vals[i]=sin(v.vals[i]);
  return w;
}

/// max as non-member function
template<int N, typename T>
const vector<N,T>& max(const vector<N,T>& u, const vector<N,T>& v) {
  if (u.l1_norm()>=v.l1_norm()) return u;
  return v;
}

/// subtraction as non-member function
template<int N, typename T>
vector<N,T> operator-(const vector<N,T>& u, const vector<N,T>& v) {
  vector<N,T> w;
  for (size_t i=0;i<N;i++) w.vals[i]=u.vals[i]-v.vals[i];
  return w;
}

/// division as non-member function
template<int N, typename T>
vector<N,T> operator/(const vector<N,T>& u, const vector<N,T>& v) {
  vector<N,T> w;
  for (size_t i=0;i<N;i++) w.vals[i]=u.vals[i]/v.vals[i];
  return w;
}

/// unary plus as non-member function
template<int N, typename T>
vector<N,T> operator+(const vector<N,T>& u) {
  return u;
}

/// arithmetic assignment as non-member function
template<int N, typename T>
vector<N,T>& operator-=(vector<N,T>& u, const vector<N,T>& v) {
  for (size_t i=0;i<N;i++) u.vals[i]-=v.vals[i];
  return u;
}

/// prefix decrement as non-member function
template<int N, typename T>
vector<N,T>& operator--(vector<N,T>& u) {
  for (size_t i=0;i<N;i++) --u.vals[i];
  return u;
}

/// postfix increment as non-member function
template<int N, typename T>
vector<N,T> operator++(vector<N,T>& u,int) {
  vector<N,T> w(u);
  for (size_t i=0;i<N;i++) u.vals[i]++;
  return w;
}

/// relational operator as non-member function
template<int N, typename T>
bool operator>=(const vector<N,T>& u, const vector<N,T>& v) {
  if (u.l1_norm()>=v.l1_norm()) return true;
  return false;
}

/// output stream only as non-member function
template<int N, typename T>
std::ostream& operator<<(std::ostream& s, const vector<N,T>& v) {
  s << "[ ";
  for (size_t i=0;i<N;i++) s << v.vals[i] << " ";
  s << "]";
  return s;
}

/// input stream only as non-member function
template<int N, typename T>
std::istream& operator>>(std::istream& s, vector<N,T>& u) {
  for (size_t i=0;i<N;i++) s >> u.vals[i];
  return s;
}

int main() {
  const int n=2;
  vector<n,float> x,y=4.2,z(y);
  x.print(); y.print(); z.print();
/*
  sin(x).print(); sin(y).print(); sin(z).print();
//
  std::cout << l2_norm(x) << std::endl
            << l2_norm(y) << std::endl
            << l2_norm(z) << std::endl;
//
  std::cout << x.l1_norm() << std::endl
            << y.l1_norm() << std::endl
            << z.l1_norm() << std::endl;
//
  max(x,y).print();
  max(y,x).print();
//
  y.print(); z.print(); 
  z=y; 
  y.print(); z.print(); 
//
  x.print(); 
  std::cin >> x[0];
  std::cout << x[0] << std::endl;
  x.print(); 
//
  (-+y).print();
  (+-z).print();
//
  (y*z).print();
  x=y/z;
  x.print();
//
  (--y).print(); (++z).print();
//
  (y++).print(); (z--).print();
  y.print(); z.print();
//
  if (y>=x) y.print(); else x.print();
//
  x+=y; y-=z;
  x.print(); y.print();
//
  std::cin >> x >> y;
  std::cout << x << y << std::endl;
*/

  return 0;
}

 

